<?php
/**
 * The admin ajax functionality of the plugin.
 */

class GOON_Ajax {

	/**
	 * Initialize the class and set its properties.
	 */
	function __construct() {

        $this->load_hooks();
	}

	/**
	 * Register all of the hooks related to the admin area functionality of the plugin.
	 *
	 * @access   private
	 */
	function load_hooks() {

		add_action( 'wp_ajax_goon_save_filter',                   [ $this, 'goon_save_filter'                   ] );
		add_action( 'wp_ajax_goon_save_group',                    [ $this, 'goon_save_group'                    ] );
		add_action( 'wp_ajax_goon_save_category',                 [ $this, 'goon_save_category'                 ] );
		add_action( 'wp_ajax_goon_create_category',               [ $this, 'goon_create_category'               ] );
		add_action( 'wp_ajax_goon_delete_elements',               [ $this, 'goon_delete_elements'               ] );
		add_action( 'wp_ajax_goon_publish_elements',              [ $this, 'goon_publish_elements'              ] );
		add_action( 'wp_ajax_goon_turn_filter_on',                [ $this, 'goon_turn_filter_on'                ] );
		add_action( 'wp_ajax_goon_turn_filter_off',               [ $this, 'goon_turn_filter_off'               ] );
		add_action( 'wp_ajax_goon_export_filters',                [ $this, 'goon_export_filters'                ] );
		add_action( 'wp_ajax_goon_import_filters',                [ $this, 'goon_import_filters'                ] );
		add_action( 'wp_ajax_goon_mark_tab_complete',             [ $this, 'goon_mark_tab_complete'             ] );
		add_action( 'wp_ajax_goon_save_option_alphabetize_menu',  [ $this, 'goon_save_option_alphabetize_menu'  ] );
		add_action( 'wp_ajax_goon_turn_off_filter',               [ $this, 'goon_turn_off_filter'               ] );
		add_action( 'wp_ajax_goon_save_original_menu',            [ $this, 'goon_save_original_menu'            ] );
		add_action( 'wp_ajax_goon_get_post_types',                [ $this, 'goon_get_post_types'                ] );
        add_action( 'wp_ajax_goon_save_columns_state',            [ $this, 'goon_save_columns_state'            ] );
        add_action( 'wp_ajax_goon_toggle_exclude_plugin',         [ $this, 'goon_toggle_exclude_plugin'         ] );
        add_action( 'wp_ajax_goon_include_all_plugins',           [ $this, 'goon_include_all_plugins'           ] );

	}

	/**
	 * Create/Update filter
	 */
	function goon_save_filter() {
        
        // goon_mu_plugin()->write_log( $_POST, "goon_save_filter-_POST" );
        
        if( empty( $_POST['data'] ) ){             wp_send_json_error( [ "message" => "The data never reached the server!" ] ); }
        
        parse_str( $_POST['data'], $array);
        
        if( empty( $array['GOON_filter_data'] ) ){ wp_send_json_error( [ "message" => "The data never reached the server!" ] ); }
        
        
        $data = GOON_Admin_Helper::format__save_filter_data( $array['GOON_filter_data'] );
        
        // goon_mu_plugin()->write_log( $data,  "goon_save_filter-data"  );
        
        if( is_wp_error( $data ) ){
            
            wp_send_json_error( [ "message" => $data->get_error_message() ] );
        }
        
        
		$post_data = array(
			'post_title'  => $data["title"],
			'post_type'   => 'goon_filter',
			'post_status' => 'publish',
			'post_author' => 1,// TODO get_current_user_id() with localize_script in enqueue function
		);
        
        if( ! empty( $data["ID"] ) ){
            $post_data["ID"] = $data["ID"];
        }

		$post_id = wp_insert_post( $post_data, true );

		if ( is_wp_error( $post_id ) ) {
			wp_send_json_error( [ "message" => $post_id->get_error_message() ] );
		}



        if( ! empty( $data["meta"]["categories"] ) ){
            
            $category_ids = array_map( 'intval', array_keys( $data["meta"]["categories"] ) );
            
            $set_categories = wp_set_object_terms( $post_id, $category_ids, "goon_categories" );
        }
        
        
        
        foreach( $data["meta"] as $meta_key => $meta_value ){
            
            update_post_meta( $post_id, $meta_key, $meta_value );
        }
        
        
        
		wp_send_json_success( [ "message" => "All good, the filter is saved.", "id" => $post_id, ] );

	}

	/**
	 * Create/Update Group
	 */
	function goon_save_group() {
        
        if( empty( $_POST['data'] ) ){              wp_send_json_error( [ "message" => "The data never reached the server!" ] ); }
        
        parse_str( $_POST['data'], $array);
        
        if( empty( $array['GOON_filter_data'] ) ){ wp_send_json_error( [ "message" => "The data never reached the server!" ] ); }
        
        
        $data = GOON_Admin_Helper::format__save_group_data( $array['GOON_filter_data'] );
        
        // goon_mu_plugin()->write_log( $_POST, "goon_save_group-_POST" );
        // goon_mu_plugin()->write_log( $data,  "goon_save_group-data"  );
        
        if( is_wp_error( $data ) ){
            
            wp_send_json_error( [ "message" => $data->get_error_message() ] );
        }
        
        
		$post_data = array(
			'post_title'  => $data["title"],
			'post_type'   => 'goon_group',
			'post_status' => 'publish',
			'post_author' => 1,// TODO get_current_user_id() with localize_script in enqueue function
		);
        
        if( ! empty( $data["ID"] ) ){
            $post_data["ID"] = $data["ID"];
        }

		$post_id = wp_insert_post( $post_data, true );

		if ( is_wp_error( $post_id ) ) {
			wp_send_json_error( [ "message" => $post_id->get_error_message() ] );
		}

        foreach( $data["meta"] as $meta_key => $meta_value ){
            
            update_post_meta( $post_id, $meta_key, $meta_value );
        }
        
        
		wp_send_json_success( [ "message" => "All good, the group is saved.", "id" => $post_id, ] );

	}

	/**
	 * Create/Update Category
	 */
	function goon_save_category() {
        
        if( empty( $_POST['data'] ) ){              wp_send_json_error( [ "message" => "The data never reached the server!" ] ); }
        
        parse_str( $_POST['data'], $array);
        
        if( empty( $array['GOON_filter_data'] ) ){ wp_send_json_error( [ "message" => "The data never reached the server!" ] ); }
        
        
        $data = GOON_Admin_Helper::format__save_category_data( $array['GOON_filter_data'] );
        
        // goon_mu_plugin()->write_log( $_POST, "goon_save_category-_POST" );
        // goon_mu_plugin()->write_log( $data,  "goon_save_category-data"  );
        
        if( is_wp_error( $data ) ){
            
            wp_send_json_error( [ "message" => $data->get_error_message() ] );
        }
        
        
        if( empty( $data["ID"] ) ){
            
            $category = wp_create_term( $data["name"], "goon_categories" );
            
            if( is_wp_error( $category ) ){
                wp_send_json_error( [ "message" => "An error occured: " . $category->get_error_message() ] );
            }
            
            $term_id = $category["term_id"];
            
        } else {
            
            $term_id = $data["ID"];
            unset( $data["ID"] );
        }
        
        $category = wp_update_term( $term_id, "goon_categories", $data );
        
        if( is_wp_error( $category ) ){
            wp_send_json_error( [ "message" => "An error occured: " . $category->get_error_message() ] );
        }
        
		wp_send_json_success( [ "message" => "All good, the category is saved.", "id" => $category["term_id"], ] );

	}

	/**
	 * Create new category
	 */
	function goon_create_category(){
        
        // goon_mu_plugin()->write_log( $_POST, "goon_create_category-_POST" );
        
        if( empty( $_POST['category_name'] ) ){
            wp_send_json_error( [ "message" => "Category name is a required field!" ] );
        }
        
        $category_name = sanitize_textarea_field( $_POST['category_name'] );
        
        if( term_exists( $category_name, "goon_categories" ) ){
            wp_send_json_error( [ "message" => "A category with that name already exists!" ] );
        }
        
        $data = wp_create_term( $category_name, "goon_categories" );
        
        if( is_wp_error( $data ) ){
            wp_send_json_error( [ "message" => "An error occured: " . $data->get_error_message() ] );
        }
        
        wp_send_json_success( [ "category_id" => $data['term_id'] ] );
	}

	/**
	 * Delete elements
	 */
	function goon_delete_elements() {
        
		$name_post_type = sanitize_textarea_field( $_POST['name_post_type'] );
		$type_elements  = sanitize_textarea_field( $_POST['type_elements'] );
        $id_elements    = array_map( 'intval', $_POST['id_elements'] );

		if ( $name_post_type === 'cat' ) {
			
			foreach ( $id_elements as $id_element ) {
				wp_delete_term( $id_element, 'goon_categories' );
			}

			wp_send_json_success( [ "message" => "Categories are deleted." ] );
            
		} elseif ( $type_elements === 'all' ) {
         
			foreach ( $id_elements as $post_id ) {
				wp_trash_post( $post_id );
			}
            
			wp_send_json_success( [ "message" => "Items are moved to trash." ] );
            
		} else {
            
			$posts = get_posts( array(
				'post_type'   => $name_post_type,
				'include'     => $id_elements,
				'post_status' => 'trash',
			) );

			foreach ( $posts as $post ) {
				wp_delete_post( $post->ID, true );
			}
            
			wp_send_json_success( [ "message" => "Items are permanently deleted." ] );
            
		}
	}

	/**
	 * Restore trash
	 */
	function goon_publish_elements() {
        
		$name_post_type = sanitize_textarea_field( $_POST['name_post_type'] );
        $id_elements    = array_map( 'intval', $_POST['id_elements'] );

		$posts = get_posts( array(
			'post_type'   => $name_post_type,
			'include'     => $id_elements,
			'post_status' => 'trash',
		) );

		foreach ( $posts as $post ) {
			wp_publish_post( $post->ID );
		}
        
		wp_send_json_success( [ "message" => "Items are restored." ] );
        
	}
    
	/**
	 * Bulk turn filters on
	 */
	function goon_turn_filter_on() {
        
		$name_post_type = sanitize_textarea_field( $_POST['name_post_type'] );
        $id_elements    = array_map( 'intval', $_POST['id_elements'] );

		$posts = get_posts( array(
			'post_type'   => $name_post_type,
			'include'     => $id_elements,
		) );

		foreach ( $posts as $post ) {
			update_post_meta( $post->ID, "turned_off", false );
		}
        
		wp_send_json_success( [ "message" => count( $posts) . " filters are turned on." ] );
        
	}
    
	/**
	 * Bulk turn filters off
	 */
	function goon_turn_filter_off() {
        
		$name_post_type = sanitize_textarea_field( $_POST['name_post_type'] );
        $id_elements    = array_map( 'intval', $_POST['id_elements'] );

		$posts = get_posts( array(
			'post_type'   => $name_post_type,
			'include'     => $id_elements,
		) );

		foreach ( $posts as $post ) {
			update_post_meta( $post->ID, "turned_off", true );
		}
        
		wp_send_json_success( [ "message" => count( $posts) . " filters are turned off." ] );
        
	}
    
	/**
	 * Bulk export filters
	 */
	function goon_export_filters() {
        
        $id_elements    = array_map( 'intval', $_POST['id_elements'] );

		$filters = get_posts( array(
			'post_type'   => 'goon_filter',
			'include'     => $id_elements,
		) );
        
        $categories_raw = get_categories( [
            'taxonomy'   => 'goon_categories',
            'type'       => 'goon_filter',
            'hide_empty' => 0,
        ] );
        
        $categories = [];
        
        foreach( $categories_raw as $category ){
            
            $categories[ $category->term_id ] = $category;
        }
        
        $export_array = [ "filters" => [], "categories" => [] ];
        
		foreach( $filters as $filter ) {
            
            $export_array["filters"][ $filter-> ID ] = [
                "title" => $filter->post_title,
                "meta"  => [
                    "filter_type"       => $filter->filter_type,
                    "endpoints"         => $filter->endpoints,
                    "plugins_to_block"  => $filter->plugins_to_block,
                    "categories"        => $filter->categories,
                    "ajax_actions"      => $filter->ajax_actions,
                    "css_files"         => $filter->css_files,
                    "js_files"          => $filter->js_files,
                ],
            ];
            
            if( ! empty( $filter->categories ) ){
                
                // goon_mu_plugin()->write_log( $filter->categories, "goon_export_filters-filter->categories" );
                
                foreach( $filter->categories as $category_id => $category ){
                    
                    $export_array["categories"][ $category_id ] = [
                        "cat_name"              => $categories[ $category_id ]->name,
                        "category_description"  => $categories[ $category_id ]->description,
                    ];
                    
                }
                
            }
            
		}
        
		wp_send_json_success( [ "message" => count( $filters ) . " filters are ready for export.", "filters" => $export_array ] );
        
	}
    
	/**
	 * Bulk import filters
	 */
	function goon_import_filters() {
        
		$data = sanitize_textarea_field( $_POST['filters'] );
        $data = stripslashes( $data );
        $data = json_decode( $data, true );
        
        // goon_mu_plugin()->write_log( $data, "goon_import_filters-data" );
        
        if( empty( $data["filters"] ) ){
            
            wp_send_json_error( [ "message" => "The import has received empty data." ] );
        }
        
        $categories_raw = get_categories( [
            'taxonomy'   => 'goon_categories',
            'type'       => 'goon_filter',
            'hide_empty' => 0,
        ] );
        
        $categories = [];
        $categories_names = [];
        
        foreach( $categories_raw as $category ){
            
            $categories[ $category->term_id ] = $category;
            $categories_names[ $category->term_id ] = $category->name;
        }
        
        // goon_mu_plugin()->write_log( $categories_raw, "goon_import_filters-categories_raw" );
        
        $imported = 0;
        
        foreach( $data["filters"] as $filter_data ){
            
            // goon_mu_plugin()->write_log( $filter_data, "goon_import_filters-filter_data" );
            
            $new_filter_data = [
                "post_type"     => "goon_filter",
                "post_status"   => "publish",
                "post_content"  => "",
                "post_title"    => wp_strip_all_tags( $filter_data["title"] ),
                "meta_input"    => $filter_data["meta"],
            ];
            
            $new_filter_data["meta_input"]["turned_off"] = true;
            $new_filter_data["meta_input"]["categories"] = [];
            
            foreach( $filter_data["meta"]["categories"] as $category_id => $category_name ){
                
                $found_category_id = array_search( $category_name, $categories_names );
                
                if( $found_category_id !== false ){
                    
                    $new_filter_data["meta_input"]["categories"][ $found_category_id ] = $category_name;
                    
                } else {
                    
                    $new_category_args = [
                        "taxonomy"              => 'goon_categories',
                        "cat_name"              => $category_name,
                        "category_description"  => $data["categories"][ $category_id ]["category_description"],
                    ];
                    
                    $new_category_id = wp_insert_category( $new_category_args );
                    
                    if( $new_category_id ){
                        
                        $new_filter_data["meta_input"]["categories"][ $new_category_id ] = $category_name;
                    }
                    
                }
                
            }
            
            
            if( wp_insert_post( $new_filter_data ) > 0 ){
                
                $imported++;
            }
            
        }
        
        $response_message = $imported > 0 ? "Success: " . sprintf( _n( "%s filter was", "%s filters were", $imported ), $imported ) . " imported.": "Failed: no filters were imported for some reason";

		wp_send_json_success( [ "message" => $response_message ] );
        
	}
    
	/**
	 * Used for the Overview page
	 */
    function goon_mark_tab_complete(){
        
        $tab_id  = sanitize_textarea_field( $_POST["tab_id"] );
        $user_id = intval( $_POST["user_id"] );
        
        $user_tabs_completed = get_user_meta( $user_id, "completed_overview_tabs", true );
        
        if( empty( $user_tabs_completed ) ){
            $user_tabs_completed = [];
        }
        
        if( ! in_array( $tab_id, $user_tabs_completed ) ){
            
            $user_tabs_completed[] = $tab_id;
            sort( $user_tabs_completed );
            
            update_user_meta( $user_id, "completed_overview_tabs", $user_tabs_completed );
        }
        
		wp_send_json_success( [ "message" => "Completed tabs are now remembered." ] );
        
    }
    
	/**
	 * From the Settings page
	 */
    function goon_save_option_alphabetize_menu(){
        
        $should  = $_POST["should_alphabetize"] === "true";
        
        update_option("goon_should_alphabetize_menu", $should, false );
        
		wp_send_json_success( [ "message" => "Option saved successfully." ] );
        
    }
    
	/**
	 * From the Filters List page
	 */
    function goon_turn_off_filter(){
        
        $turned_off  = $_POST["turned_off"] === "true";
        $post_id     = intval( $_POST["post_id"] );
        
        update_post_meta( $post_id, "turned_off", $turned_off );
        
		wp_send_json_success( [ "message" => "Filter turned " . ( $turned_off ? "off" : "on" ) . " successfully." ] );
        
    }
    
	/**
	 * Save the admin menu sidebar HTML in the database
	 */
    function goon_save_original_menu(){
        
        $menu_html          = wp_kses_post( stripcslashes( $_POST["menu_html"]        ) );
        $topbar_menu_html   = wp_kses_post( stripcslashes( $_POST["topbar_menu_html"] ) );
        $new_html           = wp_kses_post( stripcslashes( $_POST["new_html"]         ) );
        
        // goon_mu_plugin()->write_log( $menu_html, "goon_save_original_menu-menu_html" );
        
        update_option( "goon_original_menu", $menu_html,        false );
        update_option( "goon_topbar_menu",   $topbar_menu_html, false );
        update_option( "goon_new_posts",     $new_html,         false );
        
		wp_send_json_success( [ "message" => "Menu saved successfully." ] );
        
    }
    
	/**
	 * Because Ajax is not being filtered, we can get a full list of post types
	 */
    function goon_get_post_types(){
        
        $post_types = [];
        
        $post_types_raw = get_post_types( [ "show_in_menu" => true ], "objects" );
        
        // goon_mu_plugin()->write_log( $post_types_raw, "goon_get_post_types-post_types_raw" );
        
        foreach( $post_types_raw as $post_type ){
            
            $post_types[ $post_type->name ] = $post_type->labels->singular_name . " (" . $post_type->name . ")";
            
        }
        
        natsort( $post_types );
        
		wp_send_json_success( [ "message" => "Post types fetched.", "post_types" => $post_types ] );
        
    }
    
    /**
     * Saves the current state of visible columns on the Filters List screen
     */
    function goon_save_columns_state(){
        
        // goon_mu_plugin()->write_log( $_POST, "goon_save_columns_state-_POST" );
        
        if( empty( $_POST['action'] ) || $_POST['action'] !== "goon_save_columns_state" ){    wp_send_json_error( [ "message" => "We have somehow triggered the wrong action!" ] ); }
        
        $data = empty( $_POST['data'] ) ? [] : $_POST['data'];
        
        // goon_mu_plugin()->write_log( $data, "goon_save_columns_state-data" );
        
        if( $data ){
            
            foreach( $data as $index => $data_item ){
                
                $data[ $index ] = sanitize_text_field( $data_item );
                
            }
            
        }
        
        
        // goon_mu_plugin()->write_log( get_current_user_id(), "goon_save_columns_state-get_current_user_id" );
        
        update_user_meta( get_current_user_id(), "goon_filter_columns", $data );
        
		wp_send_json_success( [ "message" => "All good, the columns are saved." ] );

    }
    
    /**
     * Adds the desired plugin id on the list of Goon Excluded Plugins
     */
    function goon_toggle_exclude_plugin(){
        
        // goon_mu_plugin()->write_log( $_POST, "goon_save_columns_state-_POST" );
        
        if( empty( $_POST['action'] ) || $_POST['action'] !== "goon_toggle_exclude_plugin" ){    wp_send_json_error( [ "message" => "We have somehow triggered the wrong action!" ] ); }
        
        $data = empty( $_POST['data'] ) ? [] : $_POST['data'];
        
        // goon_mu_plugin()->write_log( $data, "goon_save_columns_state-data" );
        
        if( empty( $data ) || empty( $data['plugin_id'] ) || ! isset( $data['included'] ) ){        wp_send_json_error( [ "message" => "Data is missing!" ] ); }
        
        $plugin_id = sanitize_text_field( $data['plugin_id'] );
        
        // goon_mu_plugin()->write_log( $data['plugin_id'], "goon_toggle_exclude_plugin-plugin_id" );
        // goon_mu_plugin()->write_log( $data['included'],  "goon_toggle_exclude_plugin-included"  );
        
        $excluded_plugins = get_option( "goon_excluded_plugins", [] );
        
        if( $data["included"] === "false" ){
            
            if( ! in_array( $plugin_id, $excluded_plugins ) ){
                
                $excluded_plugins[] = $plugin_id;
                update_option( "goon_excluded_plugins", $excluded_plugins, false );
            }
            
        } else {
                    
            if( in_array( $plugin_id, $excluded_plugins ) ){
                
                $excluded_plugins = array_diff( $excluded_plugins, [ $plugin_id ] );
                update_option( "goon_excluded_plugins", $excluded_plugins, false );
            }
            
        }
        
		wp_send_json_success( [ "message" => "All good, the Goon Excluded Plugins list is up to date." ] );

    }
    
    /**
     * Deletes all plugins from the Goon Exclude Plugins list
     */
    function goon_include_all_plugins(){
        
        // goon_mu_plugin()->write_log( $_POST, "goon_save_columns_state-_POST" );
        
        if( empty( $_POST['action'] ) || $_POST['action'] !== "goon_include_all_plugins" ){    wp_send_json_error( [ "message" => "We have somehow triggered the wrong action!" ] ); }
        
        update_option( "goon_excluded_plugins", [], false );
        
		wp_send_json_success( [ "message" => "All good, the Goon Excluded Plugins list is now empty." ] );

    }
    
}

