<?php

/**
 * Class GOON_Admin_Helper
 */
class GOON_Admin_Helper
{
    static function sort__by_title( $a, $b )
    {
        return strnatcasecmp( $a->title, $b->title );
    }
    
    static function sort__by_post_title( $a, $b )
    {
        return strnatcasecmp( $a->post_title, $b->post_title );
    }
    
    static function sort__by_cat_name( $a, $b )
    {
        return strnatcasecmp( $a->cat_name, $b->cat_name );
    }
    
    static function get_filter_endpoints( $filter, $add_home = false )
    {
        $endpoints = get_post_meta( $filter->ID, "endpoints", true );
        if ( !empty($endpoints) && is_array( $endpoints ) ) {
            foreach ( $endpoints as $key => $endpoint ) {
                
                if ( $add_home ) {
                    
                    if ( strpos( $endpoint, home_url() ) === false ) {
                        $endpoints[$key] = home_url() . $endpoint;
                    } else {
                        $endpoints[$key] = $endpoint;
                    }
                
                } else {
                    $endpoints[$key] = str_replace( home_url(), "", $endpoint );
                }
            
            }
        }
        return ( !empty($endpoints) ? $endpoints : [] );
    }
    
    static function get_filter_ajax_actions( $filter )
    {
        $ajax_actions = get_post_meta( $filter->ID, "ajax_actions", true );
        return ( !empty($ajax_actions) ? $ajax_actions : [] );
    }
    
    static function get_filter_css_files( $filter )
    {
        $css_files = get_post_meta( $filter->ID, "css_files", true );
        return ( !empty($css_files) ? $css_files : [] );
    }
    
    static function get_filter_js_files( $filter )
    {
        $js_files = get_post_meta( $filter->ID, "js_files", true );
        return ( !empty($js_files) ? $js_files : [] );
    }
    
    static function content_part__header( $page_title, $class = "default" )
    {
        $tabs = [
            "Filters"    => "admin.php?page=goon_filters",
            "Categories" => "admin.php?page=goon_filters_categories",
            "Groups"     => "admin.php?page=goon_groups",
            "Settings"   => "admin.php?page=goon_settings",
        ];
        $tabs_html = '';
        foreach ( $tabs as $tab_name => $tab_href ) {
            $tab_name_low = strtolower( $tab_name );
            $tabs_html .= '<a href="' . esc_url( $tab_href ) . '" id="window_' . esc_attr( $tab_name_low ) . '"    class="tabs' . (( $class == $tab_name_low ? " current" : "" )) . '">' . $tab_name . '</a>';
        }
        $class = esc_attr( $class );
        echo  <<<EOF
        
        <div id="main_title">
            <h1></h1>
            <h2 id="name_page" class="{$class}">{$page_title}</h2>
        </div>

        <div id="main_tab_navigation" class="wrap-tabs">
            {$tabs_html}
        </div>
        
EOF
 ;
    }
    
    static function content_part__bulk_actions( $include_turn_on = false, $allow_trash = true )
    {
        $turn_on_html = "";
        
        if ( $include_turn_on ) {
            $turn_on_html .= '<option value="turn_on">Turn On</option>';
            $turn_on_html .= '<option value="turn_off">Turn Off</option>';
        }
        
        $trash_label = ( $allow_trash ? "Trash" : "Permanently Delete" );
        echo  <<<EOF
        
                <div id="bulk_actions">
                    <select id="check_all_elements">
                        <option value="default">Bulk actions</option>
                        {$turn_on_html}
                        <option value="delete">{$trash_label}</option>
                    </select>
                    <button id="btn_apply" class="goon_secondary_button">Apply</button>
                </div>
EOF
 ;
    }
    
    static function content_part__manipulate_filter_options()
    {
        echo  <<<EOF
        
                <div id="show_filter_options" class="manipulate_filter_options">
                    Show Filtering Options
                </div>
                <div id="hide_filter_options" class="manipulate_filter_options">
                    Hide Filtering Options
                </div>
EOF
 ;
    }
    
    static function content_part__manipulate_toggle_columns()
    {
        echo  <<<EOF
        
                <div id="show_toggle_columns" class="manipulate_toggle_columns">
                    Show Toggle Columns
                </div>
                <div id="hide_toggle_columns" class="manipulate_toggle_columns">
                    Hide Toggle Columns
                </div>
                
EOF
 ;
    }
    
    static function content_part__filter_options( $posts )
    {
        $months = [];
        $months_html = '';
        $filter_triggers = [];
        $filter_trigger_html = '';
        $filter_state_html = '';
        $filter_type_html = '';
        foreach ( $posts as $post ) {
            $date = $post->post_date;
            $date_value = date( "Ym", strtotime( $date ) );
            // 202109
            $date_label = date( "F Y", strtotime( $date ) );
            // September 2021
            $months[$date_value] = $date_label;
            if ( $post->post_type == "goon_filter" && !in_array( $post->filter_trigger, array_keys( $filter_triggers ) ) ) {
                
                if ( $post->filter_type == "_endpoint" ) {
                    $filter_triggers["_endpoint"] = "Endpoint";
                } elseif ( $post->filter_type == "_endpoint_excluded" ) {
                    $filter_triggers["_endpoint_excluded"] = "Endpoint Excluded";
                } elseif ( $post->filter_type == "_ajax_action" ) {
                    $filter_triggers["_ajax_action"] = "Ajax action";
                } else {
                    $filter_triggers["_edit_screen"] = "Post type";
                }
            
            }
            // goon_mu_plugin()->write_log( $post->filter_trigger, "content_part__filter_options-post-filter_trigger" );
            // break;
        }
        ksort( $months );
        foreach ( $months as $value => $label ) {
            $months_html .= '<option value="' . $value . '">' . $label . '</option>';
        }
        // only if we're on the Filters List page
        
        if ( count( $filter_triggers ) >= 2 ) {
            $filter_trigger_html .= '<div>';
            $filter_trigger_html .= '<select id="filter_by_trigger" class="filter_select">';
            $filter_trigger_html .= '    <option value="default">All triggers</option>';
            foreach ( $filter_triggers as $filter_trigger => $filter_trigger_label ) {
                $filter_trigger_html .= '<option value="' . esc_attr( $filter_trigger ) . '">' . $filter_trigger_label . '</option>';
            }
            $filter_trigger_html .= '</select>';
            $filter_trigger_html .= '</div>';
        }
        
        $filter_state_html .= '<div>';
        $filter_state_html .= '<select id="filter_by_state" class="filter_select">';
        $filter_state_html .= '    <option value="default">All states</option>';
        $filter_state_html .= '<option value="turned_on">' . 'Turned On' . '</option>';
        $filter_state_html .= '<option value="turned_off">' . 'Turned Off' . '</option>';
        $filter_state_html .= '</select>';
        $filter_state_html .= '</div>';
        echo  <<<EOF
        
                <div id="filter_options" class="toggle_filter_options">
                    <div>
                        Filtering options:
                    </div>
                    {$filter_type_html}
                    {$filter_trigger_html}
                    <div>
                        <select id="filter_by_date" class="filter_select">
                            <option value="default">All dates</option>
                            {$months_html}
                        </select>
                    </div>
                    {$filter_state_html}
                    <div id="clear_filter_options" class="manipulate_filter_options">
                        Clear Filtering Options
                    </div>
                </div>
                <script>jQuery('#filter_options').hide();</script>
            
EOF
 ;
    }
    
    static function content_part__toggle_columns_options()
    {
        $column_names = [
            'status'          => "Status",
            'title'           => "Title",
            'categories'      => "Categories",
            'triggers'        => "Triggers",
            'plugins_tooltip' => "Blocking",
            'created'         => "Created",
            'modified'        => "Modified",
            'toggle_filter'   => "Turned On",
        ];
        $column_order = [
            30 => 'categories',
            40 => 'triggers',
            60 => 'plugins_tooltip',
            90 => 'toggle_filter',
        ];
        ksort( $column_order );
        // goon_mu_plugin()->write_log( $column_order, "content_part__toggle_columns_options-column_order" );
        $hidden_columns = get_user_meta( get_current_user_id(), "goon_filter_columns", true );
        // goon_mu_plugin()->write_log( $hidden_columns, "content_part__toggle_columns_options-hidden_columns" );
        $columns = [];
        $column_html = "";
        foreach ( $column_order as $column_id ) {
            
            if ( in_array( $column_id, array_keys( $column_names ) ) ) {
                $columns[$column_id] = $column_names[$column_id];
                $column_name = $column_names[$column_id];
                $checked = ( is_array( $hidden_columns ) && in_array( $column_id, $hidden_columns ) ? '' : ' checked="checked"' );
                $column_html .= <<<EOF
                
                        <div class="single_column">
                            <div class="single_column_name">
                                {$column_names[$column_id]}
                            </div>
                            <div class="single_column_state">
                                <label>
                                    <span class="switch">
                                        <input name="{$column_id}" data-id="{$column_id}" type="checkbox"{$checked}/>
                                        <span class="slider round"></span>
                                    </span>
                                </label>
                            </div>
                        </div>
                        
EOF;
            }
        
        }
        // goon_mu_plugin()->write_log( $columns, "content_part__toggle_columns_options-columns" );
        echo  <<<EOF
        
                <div id="toggle_columns" class="toggle_columns_options">
                    <div id="full_columns_list">
                        {$column_html}
                    </div>
                    <div>
                        <div id="show_all_columns" class="manipulate_toggle_columns">Show all columns</div>
                        <div id="hide_all_columns" class="manipulate_toggle_columns">Hide all columns</div>
                        <button id="save_columns_state" class="goon_green_button">Save</button>
                    </div>
                </div>
                <script>jQuery('#toggle_columns').hide();</script>
            
EOF
 ;
    }
    
    static function content_part__plugins( $args = array() )
    {
        $defaults = [
            'plugins'  => [],
            'blocked'  => [],
            'inactive' => [],
        ];
        $data = wp_parse_args( $args, $defaults );
        // goon_mu_plugin()->write_log( $data, "content_part__plugins-data" );
        
        if ( $data["plugins"] ) {
            natsort( $data["plugins"] );
            echo  '<div class="special_grid_list">' ;
            foreach ( $data["plugins"] as $plugin_id => $plugin_name ) {
                $class = "single_plugin";
                $checked = '';
                
                if ( !empty($data["blocked"][$plugin_id]) || in_array( $plugin_id, $data["blocked"] ) ) {
                    $class .= " blocked";
                    $checked = ' checked="checked"';
                }
                
                if ( !empty($data["inactive"][$plugin_id]) || in_array( $plugin_id, $data["inactive"] ) ) {
                    $class .= " inactive";
                }
                echo  '<div class="' . esc_attr( $class ) . '" data-id="' . esc_attr( $plugin_id ) . '" data-name="' . esc_attr( $plugin_name ) . '">' ;
                echo  '<input class="noeyes" type="checkbox" name="GOON_filter_data[plugins_to_block][' . esc_attr( $plugin_id ) . ']" value="' . esc_attr( $plugin_name ) . '"' . $checked . '/>' ;
                echo  '<span class="plugin_name" value="' . esc_attr( $plugin_id ) . '">' . $plugin_name . '</span>' ;
                echo  '<span class="group_name">' . '</span>' ;
                echo  '</div>' ;
            }
            echo  '</div>' ;
        } else {
            echo  '<div>' ;
            echo  '<span>No plugins found</span>' ;
            echo  '</div>' ;
        }
    
    }
    
    static function list_content__filters( $filters )
    {
        
        if ( $filters ) {
            foreach ( $filters as $filter ) {
                $data_type = get_post_meta( $filter->ID, 'filter_type', true );
                $blocking_plugins = get_post_meta( $filter->ID, 'plugins_to_block', true );
                $turned_off = get_post_meta( $filter->ID, 'turned_off', true );
                $data_endpoints = self::get_filter_endpoints( $filter, true );
                sort( $blocking_plugins );
                
                if ( empty($data_type) || in_array( $data_type, [ "_endpoint", "_endpoint_excluded" ] ) || $data_type == "none" ) {
                    $trigger_title = implode( PHP_EOL, $data_endpoints );
                    $trigger = '<b>';
                    $trigger .= ( $data_type == "_endpoint_excluded" ? "All endpoints except:" : "Endpoints:" );
                    $trigger .= '</b><br/>';
                    $trigger .= implode( ',<br>', $data_endpoints );
                    $type = ( $data_type == "_endpoint_excluded" ? "_endpoint_excluded" : "_endpoint" );
                } else {
                    
                    if ( $data_type == "_ajax_action" ) {
                    } else {
                        $trigger_title = "Editing post type: " . $data_type;
                        $trigger = "<b>Editing post type: </b>" . $data_type;
                        $type = "_edit_screen";
                    }
                
                }
                
                $categories = get_post_meta( $filter->ID, 'categories', true );
                $categories = ( !empty($categories) ? implode( ',<br>', $categories ) : "" );
                $date = date( "Ym", strtotime( $filter->post_date ) );
                // 202109
                $turned_on_checked = ( $turned_off !== "1" ? ' checked="checked"' : '' );
                $has_tooltip_class = "";
                $tooltip_list = "";
                
                if ( count( $blocking_plugins ) > 0 ) {
                    $has_tooltip_class = "has_tooltip";
                    $tooltip_list = 'class="tooltip_trigger" data-tooltip-list="' . esc_attr( htmlspecialchars( json_encode( $blocking_plugins ), ENT_QUOTES, 'UTF-8' ) ) . '"';
                }
                
                ?>
                <tr class="block_info" id="filter-<?php 
                echo  esc_attr( $filter->ID ) ;
                ?>" data-status="<?php 
                echo  esc_attr( $filter->post_status ) ;
                ?>" data-date="<?php 
                echo  esc_attr( $date ) ;
                ?>" data-type="<?php 
                echo  esc_attr( $type ) ;
                ?>">
                    <td data-label="checkbox"><input type="checkbox" class="main_selector" id="<?php 
                echo  esc_attr( $filter->ID ) ;
                ?>"></td>
                    <td data-label="title" class="align-left normal-text">
                        <?php 
                echo  $filter->post_title ;
                ?>
                        <br/>
                            <a class="edit_item" href="<?php 
                echo  esc_url( admin_url( 'admin.php?page=goon_add_filters&filter_id=' . $filter->ID ) ) ;
                ?>">Edit</a>
                        <br/>
                    </td>
                    <td data-label="categories" class="align-left normal-text"><?php 
                echo  $categories ;
                ?></td>
                    <td data-label="triggers" class="align-left normal-text" title="<?php 
                echo  esc_attr( $trigger_title ) ;
                ?>"><?php 
                echo  $trigger ;
                ?></td>
                    <td data-label="plugins_tooltip" class="list_of_plugins <?php 
                echo  esc_attr( $has_tooltip_class ) ;
                ?>">
                        <span <?php 
                echo  $tooltip_list ;
                ?>><?php 
                echo  count( $blocking_plugins ) ;
                ?></span>
                    </td>
                    <td class="toggle_filter">
                        <label>
                            <span class="switch">
                                <input class="turn_off_filter" data-id="<?php 
                echo  esc_attr( $filter->ID ) ;
                ?>" type="checkbox"<?php 
                echo  $turned_on_checked ;
                ?>/>
                                <span class="slider round"></span>
                            </span>
                        </label>
                    </td>
                </tr>
			<?php 
            }
        } else {
            ?>
            <tr>
                <td colspan="6">No filters found</td>
            </tr>
		<?php 
        }
    
    }
    
    static function list_content__groups( $groups )
    {
        
        if ( $groups ) {
            foreach ( $groups as $group ) {
                $group_plugins = get_post_meta( $group->ID, 'group_plugins', true );
                $date = date( "Ym", strtotime( $group->post_date ) );
                // 202109
                ?>
                <tr class="block_info" id="group_<?php 
                echo  esc_attr( $group->ID ) ;
                ?>" data-status="<?php 
                echo  esc_attr( $group->post_status ) ;
                ?>" data-date="<?php 
                echo  esc_attr( $date ) ;
                ?>">
                    <td data-label="checkbox"><input type="checkbox" class="main_selector" id="<?php 
                echo  $group->ID ;
                ?>"></td>
                    <td data-label="title" class="align-left normal-text"><?php 
                echo  $group->post_title ;
                ?><br/><a class="edit_item" href="<?php 
                echo  esc_url( admin_url( 'admin.php?page=goon_add_groups&group_id=' . $group->ID ) ) ;
                ?>">Edit</a><br/></td>
                    <td data-label="plugins"><?php 
                echo  implode( '<br/>', $group_plugins ) ;
                ?></td>
                    <td data-label="count"><?php 
                echo  ( $group_plugins ? count( $group_plugins ) : 0 ) ;
                ?></td>
                </tr>
			<?php 
            }
        } else {
            ?>
            <tr>
                <td colspan="5">No Groups found</td>
            </tr>
		<?php 
        }
    
    }
    
    static function list_content__categories( $categories )
    {
        
        if ( $categories ) {
            foreach ( $categories as $cat ) {
                ?>
                <tr class="block_info" id="cat-<?php 
                echo  esc_attr( $cat->term_id ) ;
                ?>" data-status="publish">
                    <td data-label="checkbox" class="cat_checkbox"><input type="checkbox" class="main_selector" id="<?php 
                echo  esc_attr( $cat->term_id ) ;
                ?>"></td>
                    <td class="cat_edit"><a href="<?php 
                echo  esc_url( admin_url( 'admin.php?page=goon_add_categories&cat_id=' . $cat->term_id ) ) ;
                ?>">Edit</a></td>
                    <td data-label="title" class="cat_title"><?php 
                echo  $cat->cat_name ;
                ?></td>
                    <td data-label="description" class="cat_description"><?php 
                echo  ( $cat->description ? $cat->description : "-" ) ;
                ?></td>
                    <?php 
                // TODO add a link to filter the filters by category
                ?>
                    <td data-label="count" class="cat_filters"><?php 
                echo  $cat->count ;
                ?></td>
                </tr>
			<?php 
            }
        } else {
            ?>
            <tr>
                <td colspan="5">No categories</td>
            </tr>
		<?php 
        }
    
    }
    
    // if $all == false -> return array( "active" => [], "inactive" => [] )
    // if $all == true  -> return []
    static function get_plugins_with_status( $all = false, $remove_goon = true )
    {
        require_once ABSPATH . 'wp-admin/includes/plugin.php';
        $plugins = [];
        $plugins_simple_list = [
            "active"   => [],
            "inactive" => [],
            "all"      => [],
        ];
        
        if ( empty(goon_mu_plugin()->all_plugins) ) {
            $all_plugins = get_plugins();
            $active_plugins = get_option( 'active_plugins' );
        } else {
            $all_plugins = goon_mu_plugin()->all_plugins;
            $active_plugins = goon_mu_plugin()->original_active_plugins;
        }
        
        // goon_mu_plugin()->write_log( $active_plugins, "get_plugins_with_status-active_plugins" );
        // goon_mu_plugin()->write_log( $all_plugins,    "get_plugins_with_status-all_plugins" );
        foreach ( $active_plugins as $plugin_id ) {
            if ( empty($all_plugins[$plugin_id]) ) {
                continue;
            }
            
            if ( !in_array( $plugin_id, goon_mu_plugin()->goon_plugins ) || !$remove_goon ) {
                $plugins_simple_list["active"][$plugin_id] = $all_plugins[$plugin_id]['Name'];
                $plugins_simple_list["all"][$plugin_id] = $all_plugins[$plugin_id]['Name'];
            }
            
            $plugins[] = [
                'name'      => $all_plugins[$plugin_id]['Name'],
                'file'      => $plugin_id,
                'is_active' => 1,
            ];
            unset( $all_plugins[$plugin_id] );
        }
        foreach ( $all_plugins as $plugin_id => $plugin_data ) {
            if ( empty($all_plugins[$plugin_id]) ) {
                continue;
            }
            
            if ( !in_array( $plugin_id, goon_mu_plugin()->goon_plugins ) || !$remove_goon ) {
                $plugins_simple_list["inactive"][$plugin_id] = $all_plugins[$plugin_id]['Name'];
                $plugins_simple_list["all"][$plugin_id] = $all_plugins[$plugin_id]['Name'];
            }
            
            $plugins[] = [
                'name'      => $plugin_data['Name'],
                'file'      => $plugin_id,
                'is_active' => 0,
            ];
        }
        return ( $all ? $plugins : $plugins_simple_list );
    }
    
    /* Validation, sanitization and formatting */
    /**
     * Format the data used to save a filter
     * 
     * The filter can be new (if we don't have the key 'ID') or an existing one.
     * This function will rearrange the array, sanitize the inputs and validate some fields.
     * 
     * @param array $data The form data sent via Ajax
     * 
     * @return array
     */
    static function format__save_filter_data( $data )
    {
        // goon_mu_plugin()->write_log( $data, "format__save_filter_data-data" );
        if ( empty($data["title"]) ) {
            return new WP_Error( 'missing_title', "The title is a required field!" );
        }
        if ( !empty($data["type"]) && $data["type"] == "_endpoint" && count( $data["endpoints"] ) === 1 && empty($data["endpoints"][0]) ) {
            return new WP_Error( 'missing_endpoint', "There has to be at least 1 endpoint defined for this filter type!" );
        }
        // basically none of the meta data needs to be sanitized because add_metadata() already does that
        $safe_data = [
            "title" => sanitize_text_field( $data["title"] ),
            "meta"  => [
            "filter_type"      => ( !empty($data["type"]) ? sanitize_text_field( $data["type"] ) : "" ),
            "endpoints"        => ( !empty($data["endpoints"]) ? $data["endpoints"] : [] ),
            "is_or_not"        => ( !empty($data["is_or_not"]) ? $data["is_or_not"] : [] ),
            "plugins_to_block" => ( !empty($data["plugins_to_block"]) ? $data["plugins_to_block"] : [] ),
            "groups_used"      => ( !empty($data["groups"]) ? $data["groups"] : [] ),
            "categories"       => ( !empty($data["categories"]) ? $data["categories"] : [] ),
            "turned_off"       => ( !empty($data["turned_on"]) ? "0" : "1" ),
        ],
        ];
        if ( !empty($data["ID"]) ) {
            $safe_data["ID"] = intval( $data["ID"] );
        }
        return $safe_data;
    }
    
    /**
     * Format the data used to save a group
     * 
     * The group can be new (if we don't have the key 'ID') or an existing one.
     * This function will rearrange the array, sanitize the inputs and validate some fields.
     * 
     * @param array $data The form data sent via Ajax
     * 
     * @return array
     */
    static function format__save_group_data( $data )
    {
        if ( empty($data["title"]) ) {
            return new WP_Error( 'missing_title', "The title is a required field!" );
        }
        if ( empty($data["plugins_to_block"]) ) {
            return new WP_Error( 'missing_title', "There has to be at least 1 plugin selected in order to save this group!" );
        }
        // basically none of the meta data needs to be sanitized because add_metadata() already does that
        $safe_data = [
            "title" => sanitize_text_field( $data["title"] ),
            "meta"  => [
            "group_plugins" => $data["plugins_to_block"],
        ],
        ];
        if ( !empty($data["ID"]) ) {
            $safe_data["ID"] = intval( $data["ID"] );
        }
        return $safe_data;
    }
    
    /**
     * Format the data used to save a category
     * 
     * The category can be new (if we don't have the key 'ID') or an existing one.
     * This function will rearrange the array, sanitize the inputs and validate some fields.
     * 
     * @param array $data The form data sent via Ajax
     * 
     * @return array
     */
    static function format__save_category_data( $data )
    {
        if ( empty($data["title"]) ) {
            return new WP_Error( 'missing_title', "The title is a required field!" );
        }
        $safe_data = [
            "name" => sanitize_text_field( $data["title"] ),
        ];
        if ( !empty($data["description"]) ) {
            $safe_data["description"] = sanitize_textarea_field( $data["description"] );
        }
        if ( !empty($data["ID"]) ) {
            $safe_data["ID"] = intval( $data["ID"] );
        }
        return $safe_data;
    }

}