<?php

/**
 * The admin-specific functionality of the plugin.
 */
class GOON_Admin
{
    /**
     * The ID of this plugin.
     *
     * @access   private
     * @var      string $plugin_name The ID of this plugin.
     */
    private  $plugin_name ;
    /**
     * The version of this plugin.
     *
     * @access   private
     * @var      string $version The current version of this plugin.
     */
    private  $version ;
    /**
     * Initialize the class and set its properties.
     *
     * @param string $plugin_name The name of this plugin.
     * @param string $version The version of this plugin.
     */
    function __construct( $plugin_name, $version )
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        // goon_mu_plugin()->write_log( $plugin_name,  "__construct-plugin_name"  );
        $this->load_hooks();
    }
    
    /**
     * Register all of the hooks related to the admin area functionality of the plugin.
     *
     * @access   private
     */
    function load_hooks()
    {
        add_filter( 'admin_body_class', [ $this, 'mark_admin_body_class' ] );
        add_action( 'admin_enqueue_scripts', [ $this, 'enqueue_styles' ] );
        add_action( 'admin_enqueue_scripts', [ $this, 'enqueue_scripts' ] );
        add_action( 'init', [ $this, 'register_post_types' ] );
        add_action( 'init', [ $this, 'register_taxonomies' ] );
        add_action( 'in_admin_header', [ $this, 'disable_all_notice_nags' ] );
        add_action( 'admin_bar_menu', [ $this, 'add_plugin_in_admin_bar' ], 100 );
        add_action(
            'upgrader_process_complete',
            [ $this, 'do_after_plugin_update' ],
            10,
            2
        );
        add_filter(
            'plugin_action_links',
            [ $this, 'plugin_action_links' ],
            10,
            2
        );
        add_filter(
            'manage_plugins_columns',
            [ $this, 'custom_plugins_columns' ],
            10,
            2
        );
        add_action(
            'manage_plugins_custom_column',
            [ $this, 'custom_plugins_columns_values' ],
            10,
            3
        );
    }
    
    function custom_plugins_columns( $columns )
    {
        // goon_mu_plugin()->write_log( $columns, "goon-custom_plugins_columns-columns" );
        $columns["goon"] = '<span id="include_all_plugins" title="Click to include all">Goon</span>';
        return $columns;
    }
    
    function custom_plugins_columns_values( $column_name, $plugin_file, $plugin_data )
    {
        $excluded = in_array( $plugin_file, goon_mu_plugin()->excluded_plugins );
        // goon_mu_plugin()->write_log( $columns, "goon-custom_plugins_columns-columns" );
        $class = ( $excluded ? "excluded" : 'included' );
        $checked = ( $excluded ? "" : ' checked="checked"' );
        echo  '<label class="goon_exclude_plugin_label ' . esc_attr( $class ) . '">' ;
        echo  '<input type="checkbox" name="' . esc_attr( $plugin_file ) . '" class="goon_exclude_plugin"' . $checked . '/>' ;
        echo  '<span class="included dashicons dashicons-yes"    title="Included - click to exclude"></span>' ;
        echo  '<span class="excluded dashicons dashicons-no-alt" title="Excluded - click to include"></span>' ;
        echo  '</label>' ;
    }
    
    function plugin_action_links( $links, $file )
    {
        
        if ( $file == 'goon-plugin-control/goon.php' || $file == 'goon-pro/goon.php' ) {
            $settings_link = '<a href="' . admin_url( 'admin.php?page=goon_settings' ) . '">Settings</a>';
            // array_unshift( $links, $settings_link );// put it first
            $links[] = $settings_link;
            // put it last
        }
        
        return $links;
    }
    
    function do_after_plugin_update( $wp_upgrader_object, $options )
    {
        // https://developer.wordpress.org/reference/hooks/upgrader_process_complete/
        // goon_mu_plugin()->write_log( $wp_upgrader_object,  "do_afet_plugin_update-wp_upgrader_object"  );
        // goon_mu_plugin()->write_log( $options,             "do_afet_plugin_update-options"             );
        // Array
        // (
        // [action] => update
        // [type] => plugin
        // [bulk] => 1
        // [plugins] => Array
        // (
        // [0] => wordpress-beta-tester/wp-beta-tester.php
        // )
        // )
        // ----
        // check if we need to convert our post types from the old names
        global  $wpdb ;
        $table = $wpdb->prefix . "posts";
        $query = "\r\n            SELECT      DISTINCT( post_type )\r\n            FROM        {$table}\r\n            WHERE       post_type IN ( 'goon_filter', 'goon_group', 'goon_work' )\r\n        ";
        $old_post_types = $wpdb->get_col( $query );
        
        if ( !empty($old_post_types) ) {
            // TODO check next 3 lines - issues when the renaming took place?
            $wpdb->query( "UPDATE  " . $wpdb->prefix . "posts         SET post_type = 'goon_filter'     WHERE post_type = 'goon_filter';" );
            $wpdb->query( "UPDATE  " . $wpdb->prefix . "posts         SET post_type = 'goon_group'      WHERE post_type = 'goon_group';" );
            $wpdb->query( "UPDATE  " . $wpdb->prefix . "posts         SET post_type = 'goon_work'       WHERE post_type = 'goon_work';" );
            $wpdb->query( "UPDATE  " . $wpdb->prefix . "term_taxonomy SET taxonomy  = 'goon_categories' WHERE taxonomy LIKE '_ategories_filters' OR taxonomy LIKE 'goon%tegories';" );
        }
        
        // put the MU plugin in place
        
        if ( !file_exists( WPMU_PLUGIN_DIR ) ) {
            mkdir( WPMU_PLUGIN_DIR );
            chmod( WPMU_PLUGIN_DIR, 0755 );
        }
        
        copy( dirname( __DIR__ ) . '/includes/class-goon-mu.php', WPMU_PLUGIN_DIR . '/class-goon-mu.php' );
        // rewrite rules
        flush_rewrite_rules( false );
        // false = soft
    }
    
    function mark_admin_body_class( $classes )
    {
        if ( function_exists( "goon_mu_plugin" ) && (count( goon_mu_plugin()->blocked_plugins ) >= 1 || get_option( "goon_should_alphabetize_menu" ) == "1") ) {
            $classes .= ' goon_is_blocking_plugins ';
        }
        if ( !empty($_GET["goon_original_menu"]) && $_GET["goon_original_menu"] == "get" ) {
            $classes .= ' goon_is_recreating_menu ';
        }
        $screen = get_current_screen();
        // write_log( $screen->base, "mark_adminh_body_class-screen" );
        if ( $screen->base === "toplevel_page_goon" ) {
            $classes .= " goon_page_overview ";
        }
        if ( strpos( $screen->base, "goon_page_goon_" ) === 0 ) {
            $classes .= " " . str_replace( "goon_page_goon_", "goon_page_", $screen->base ) . " ";
        }
        return $classes;
    }
    
    /**
     * Register the stylesheets for the admin area.
     */
    function enqueue_styles()
    {
        wp_enqueue_style(
            $this->plugin_name . '-public',
            plugin_dir_url( __FILE__ ) . 'css/goon-admin-public.css',
            array(),
            $this->version,
            'all'
        );
        wp_enqueue_style(
            $this->plugin_name . '-adminbar',
            plugin_dir_url( __DIR__ ) . 'css/adminbar.css',
            array(),
            $this->version,
            'all'
        );
        wp_enqueue_style(
            $this->plugin_name . '-fs_connect',
            plugin_dir_url( __FILE__ ) . 'css/fs_connect.css',
            array( 'fs_connect' ),
            $this->version,
            'all'
        );
        
        if ( stripos( str_replace( [ 'goon_preview_filter', 'goon_unfiltered_mode' ], '', $_SERVER["QUERY_STRING"] ), "goon" ) ) {
            wp_enqueue_style(
                $this->plugin_name,
                plugin_dir_url( __FILE__ ) . 'css/goon-admin.css',
                array(),
                $this->version,
                'all'
            );
            wp_enqueue_style(
                $this->plugin_name . '-simplebar',
                plugin_dir_url( __FILE__ ) . 'css/simplebar.css',
                array(),
                $this->version,
                'all'
            );
            wp_enqueue_style(
                $this->plugin_name . '-selectable',
                plugin_dir_url( __FILE__ ) . 'css/selectable-ui.min.css',
                array(),
                $this->version,
                'all'
            );
        }
    
    }
    
    /**
     * Register the JavaScript for the admin area.
     */
    function enqueue_scripts()
    {
        $array = array(
            'admin_url'  => admin_url(),
            'ajax_url'   => admin_url( 'admin-ajax.php' ),
            'home_url'   => home_url(),
            'plugin_url' => plugin_dir_url( __DIR__ ),
            'user_id'    => get_current_user_id(),
        );
        // get the menu from the database
        
        if ( function_exists( "goon_mu_plugin" ) && (count( goon_mu_plugin()->blocked_plugins ) >= 1 || get_option( "goon_should_alphabetize_menu" ) == "1") ) {
            $original_menu = get_option( "goon_original_menu" );
            if ( $original_menu ) {
                $array["original_menu"] = $original_menu;
            }
        }
        
        // get the topbar from the database
        
        if ( function_exists( "goon_mu_plugin" ) && count( goon_mu_plugin()->blocked_plugins ) >= 1 ) {
            $topbar_menu = get_option( "goon_topbar_menu" );
            $new_posts = get_option( "goon_new_posts" );
            if ( $topbar_menu ) {
                $array["topbar_menu"] = $topbar_menu;
            }
            if ( $new_posts ) {
                $array["new_posts"] = $new_posts;
            }
        }
        
        // if Freemius activation is completed, we ARE getting the original menu
        
        if ( !goon_fs()->is_activation_mode() && get_option( "goon_should_recreate_menu" ) === "1" ) {
            update_option( "goon_should_recreate_menu", false, false );
            $current_url = goon_mu_plugin()->current_full_url;
            $recreate_menu_url = $current_url . (( strpos( $current_url, '?' ) !== false ? '&' : '?' )) . 'goon_original_menu=get&redirect_to=' . urlencode( $current_url );
            if ( wp_redirect( $recreate_menu_url ) ) {
                exit;
            }
        }
        
        // are we getting the original menu
        
        if ( !empty($_GET["goon_original_menu"]) && $_GET["goon_original_menu"] == "get" ) {
            $version = date( "ymd-Gis", filemtime( plugin_dir_path( __FILE__ ) . 'js/goon-admin-menu-get.js' ) );
            wp_register_script(
                $this->plugin_name . "_menu_get",
                plugin_dir_url( __FILE__ ) . 'js/goon-admin-menu-get.js',
                array( 'jquery' ),
                $version,
                false
            );
            wp_enqueue_script( $this->plugin_name . "_menu_get" );
            if ( !empty($_GET["redirect_to"]) ) {
                $array["redirect_to"] = esc_url_raw( $_GET["redirect_to"] );
            }
            if ( get_option( "goon_should_alphabetize_menu" ) === "1" ) {
                $array["alphabetize_menu"] = true;
            }
            // or are we fixing the current menu
        } else {
            $version = date( "ymd-Gis", filemtime( plugin_dir_path( __FILE__ ) . 'js/goon-admin-menu-fix.js' ) );
            wp_register_script(
                $this->plugin_name . "_menu_fix",
                plugin_dir_url( __FILE__ ) . 'js/goon-admin-menu-fix.js',
                array( 'jquery' ),
                $version,
                false
            );
            wp_enqueue_script( $this->plugin_name . "_menu_fix" );
        }
        
        $version = date( "ymd-Gis", filemtime( plugin_dir_path( __FILE__ ) . 'js/simplebar.min.js' ) );
        wp_register_script(
            $this->plugin_name . '-simplebar',
            plugin_dir_url( __FILE__ ) . 'js/simplebar.min.js',
            array( 'jquery' ),
            $version,
            true
        );
        wp_enqueue_script( $this->plugin_name . '-simplebar' );
        $version = date( "ymd-Gis", filemtime( plugin_dir_path( __FILE__ ) . 'js/selectable-ui.min.js' ) );
        wp_register_script(
            $this->plugin_name . '-selectable',
            plugin_dir_url( __FILE__ ) . 'js/selectable-ui.min.js',
            array( 'jquery' ),
            $version,
            true
        );
        wp_enqueue_script( $this->plugin_name . '-selectable' );
        $version = date( "ymd-Gis", filemtime( plugin_dir_path( __FILE__ ) . 'js/goon-admin.js' ) );
        wp_register_script(
            $this->plugin_name,
            plugin_dir_url( __FILE__ ) . 'js/goon-admin.js',
            array( 'jquery', $this->plugin_name . '-selectable' ),
            $version,
            true
        );
        wp_localize_script( $this->plugin_name, 'goon_object', $array );
        wp_enqueue_script( $this->plugin_name );
    }
    
    function check_memory_usage()
    {
        return ( function_exists( 'memory_get_peak_usage' ) ? round( memory_get_peak_usage() / 1024 / 1024, 2 ) : 0 );
    }
    
    /**
     * Add Admin-Bar Pages
     */
    function add_plugin_in_admin_bar( $wp_admin_bar )
    {
        $current_url = goon_mu_plugin()->current_full_url;
        $main_title_class = ( goon_mu_plugin()->is_being_filtered ? 'goon_bar_green' : 'goon_bar_blue' );
        $main_title_class = ( goon_mu_plugin()->is_preview_mode ? 'goon_bar_yellow' : $main_title_class );
        $main_title_class = ( goon_mu_plugin()->is_unfiltered_mode ? 'goon_bar_yellow' : $main_title_class );
        $main_title = "Goon";
        $main_title = $this->check_memory_usage() . 'MB | ' . round( microtime( true ) - goon_mu_plugin()->start_time, 2 ) . 's';
        // Main top menu item
        $wp_admin_bar->add_menu( array(
            'id'    => 'goon',
            'title' => '<span class="' . $main_title_class . '"><span class="goon-icon"></span> ' . $main_title . '</span>',
            'href'  => esc_url( get_admin_url( null, 'admin.php?page=goon_filters' ) ),
            'meta'  => [
            'class' => $main_title_class,
        ],
        ) );
        // Memory used
        $wp_admin_bar->add_menu( array(
            'parent' => 'goon',
            'id'     => 'goon_memory_used',
            'title'  => '<span>Memory used: </span><span>' . $this->check_memory_usage() . 'MB</span>',
        ) );
        // Loading time
        $wp_admin_bar->add_menu( array(
            'parent' => 'goon',
            'id'     => 'goon_loading_time',
            'title'  => '<span>Request time: </span><span>' . round( microtime( true ) - goon_mu_plugin()->start_time, 2 ) . 's</span>',
        ) );
        if ( goon_mu_plugin()->is_goon_default_page ) {
            $wp_admin_bar->add_menu( array(
                'parent' => 'goon',
                'id'     => 'goon_default_page',
                'title'  => '<span class="add_little_right_padding">We are on a Goon default page</span>',
            ) );
        }
        
        if ( !goon_mu_plugin()->is_being_filtered ) {
            // write_log( wp_title('', false ), "agsaetrz23w45q4332-wp_title" );
            $wp_title = wp_title( '', false );
            $wp_title = ( empty($wp_title) ? 'New Filter' : $wp_title );
            $admin_relative_url = 'admin.php?page=goon_add_filters';
            $admin_relative_url .= '&new_title=';
            $admin_relative_url .= urlencode( $wp_title );
            $admin_relative_url .= '&new_link=';
            $admin_relative_url .= esc_url( wp_make_link_relative( home_url( $_SERVER['REQUEST_URI'] ) ) );
            $create_link = get_admin_url( null, $admin_relative_url );
            $wp_admin_bar->add_menu( array(
                'parent' => 'goon',
                'id'     => 'goon_being_filtered',
                'title'  => '<span class="add_little_right_padding">This page is not being filtered</span>',
            ) );
            $wp_admin_bar->add_menu( array(
                'parent' => 'goon_being_filtered',
                'id'     => 'goon_being_filtered_create_new',
                'title'  => 'Create new filter',
                'href'   => $create_link,
            ) );
        }
        
        // Filters in use
        
        if ( !empty(goon_mu_plugin()->filters_in_use) ) {
            $wp_admin_bar->add_menu( array(
                'parent' => 'goon',
                'id'     => 'goon_filters_in_use',
                'title'  => 'Filters in use: ' . count( goon_mu_plugin()->filters_in_use ),
            ) );
            foreach ( goon_mu_plugin()->filters_in_use as $filter_id => $filter_name ) {
                $wp_admin_bar->add_menu( array(
                    'parent' => 'goon_filters_in_use',
                    'id'     => 'goon_filter_in_use_' . $filter_id,
                    'title'  => $filter_name,
                    'href'   => admin_url( 'admin.php?page=goon_add_filters&filter_id=' . $filter_id ),
                ) );
            }
        }
        
        // Blocked Plugins
        $wp_admin_bar->add_menu( array(
            'parent' => 'goon',
            'id'     => 'goon_blocked_plugins',
            'title'  => 'Blocked Plugins (' . count( goon_mu_plugin()->blocked_plugins ) . ')',
        ) );
        foreach ( goon_mu_plugin()->get_names_list( "blocked_plugins" ) as $plugin_path => $plugin_name ) {
            $wp_admin_bar->add_menu( array(
                'parent' => 'goon_blocked_plugins',
                'id'     => 'goon_blocked_plugin_' . $plugin_path,
                'title'  => $plugin_name,
            ) );
        }
        // Running Plugins
        $wp_admin_bar->add_menu( array(
            'parent' => 'goon',
            'id'     => 'goon_running_plugins',
            'title'  => 'Running Plugins (' . count( goon_mu_plugin()->filtered_active_plugins ) . ')',
        ) );
        foreach ( goon_mu_plugin()->get_names_list( "filtered_active_plugins" ) as $plugin_path => $plugin_name ) {
            $wp_admin_bar->add_menu( array(
                'parent' => 'goon_running_plugins',
                'id'     => 'goon_running_plugin_' . $plugin_path,
                'title'  => $plugin_name,
            ) );
        }
        // Blocked CSS
        
        if ( !empty(goon_mu_plugin()->blocked_css) ) {
            $wp_admin_bar->add_menu( array(
                'parent' => 'goon',
                'id'     => 'goon_blocked_css',
                'title'  => 'Blocked CSS: ' . count( goon_mu_plugin()->blocked_css ),
            ) );
            ksort( goon_mu_plugin()->blocked_css );
            foreach ( goon_mu_plugin()->blocked_css as $handle => $src ) {
                $file_name = basename( parse_url( $src, PHP_URL_PATH ) );
                $wp_admin_bar->add_menu( array(
                    'parent' => 'goon_blocked_css',
                    'id'     => 'goon_blocked_css_' . $handle,
                    'title'  => '<span style="color: #fff;">' . $handle . '</span>' . " " . '<span>' . $file_name . '</span>',
                    'href'   => $src,
                ) );
            }
        }
        
        // Blocked JS
        
        if ( !empty(goon_mu_plugin()->blocked_js) ) {
            $wp_admin_bar->add_menu( array(
                'parent' => 'goon',
                'id'     => 'goon_blocked_js',
                'title'  => 'Blocked JS: ' . count( goon_mu_plugin()->blocked_js ),
            ) );
            ksort( goon_mu_plugin()->blocked_js );
            foreach ( goon_mu_plugin()->blocked_js as $handle => $src ) {
                $file_name = basename( parse_url( $src, PHP_URL_PATH ) );
                $wp_admin_bar->add_menu( array(
                    'parent' => 'goon_blocked_js',
                    'id'     => 'goon_blocked_js_' . $handle,
                    'title'  => '<span style="color: #fff;">' . $handle . '</span>' . " " . '<span>' . $file_name . '</span>',
                    'href'   => $src,
                ) );
            }
        }
        
        // Settings
        $wp_admin_bar->add_menu( array(
            'parent' => 'goon',
            'id'     => 'goon_settings',
            'title'  => 'Settings',
            'href'   => admin_url( 'admin.php?page=goon_settings' ),
        ) );
        // Recreate the menu
        $wp_admin_bar->add_menu( array(
            'parent' => 'goon',
            'id'     => 'goon_recreate_the_menu',
            'title'  => 'Recreate the menu',
            'href'   => $current_url . (( strpos( $current_url, '?' ) !== false ? '&' : '?' )) . 'goon_original_menu=get&redirect_to=' . urlencode( $current_url ),
        ) );
        // Temp turn filtering off
        if ( goon_mu_plugin()->is_being_filtered && !goon_mu_plugin()->is_unfiltered_mode ) {
            $wp_admin_bar->add_menu( array(
                'parent' => 'goon',
                'id'     => 'goon_unfiltered_page',
                'title'  => 'Visit in Unfiltered Mode',
                'href'   => $current_url . (( strpos( $current_url, '?' ) !== false ? '&' : '?' )) . 'goon_unfiltered_mode=yes',
            ) );
        }
        // Turn filtering back on
        if ( goon_mu_plugin()->is_unfiltered_mode ) {
            $wp_admin_bar->add_menu( array(
                'parent' => 'goon',
                'id'     => 'goon_unfiltered_page',
                'title'  => 'Turn off Unfiltered Mode',
                'href'   => $current_url,
                'meta'   => [
                'class' => 'color_highlight',
            ],
            ) );
        }
        
        if ( !goon_mu_plugin()->is_goon_default_page ) {
            
            if ( goon_mu_plugin()->is_preview_mode ) {
                $url = $current_url;
                $title = 'Turn off Preview Mode';
                $class = 'color_highlight';
            } else {
                $url = $current_url . (( strpos( $current_url, '?' ) !== false ? '&' : '?' )) . 'goon_preview_filter=yes';
                $title = 'Visit in Preview Mode';
                $class = '';
            }
            
            // Turn on/off Preview Mode
            $wp_admin_bar->add_menu( array(
                'parent' => 'goon',
                'id'     => 'goon_preview_mode',
                'title'  => '<span class="add_little_right_padding">' . $title . '</span>',
                'href'   => $url,
                'meta'   => [
                'class' => $class,
            ],
            ) );
        }
        
        
        if ( !goon_mu_plugin()->is_goon_default_page ) {
            // New filter
            $wp_title = wp_title( '', false );
            $wp_title = ( empty($wp_title) ? 'New Filter' : $wp_title );
            $admin_relative_url = 'admin.php?page=goon_add_filters';
            $admin_relative_url .= '&new_title=' . urlencode( $wp_title );
            $admin_relative_url .= '&new_link=' . esc_url( wp_make_link_relative( home_url( $_SERVER['REQUEST_URI'] ) ) );
            $create_link = get_admin_url( null, $admin_relative_url );
            $wp_admin_bar->add_menu( array(
                'parent' => 'goon',
                'id'     => 'goon_new_filter',
                'title'  => '<span class="add_little_right_padding">Create new filter for this page</span>',
                'href'   => $create_link,
            ) );
        }
    
    }
    
    /**
     * Register all post types
     */
    function register_post_types()
    {
        /**
         * Register filter for page
         */
        register_post_type( 'goon_filter', array(
            'label'               => null,
            'labels'              => array(
            'name'               => 'Filters',
            'singular_name'      => 'Filter',
            'add_new'            => 'Add Filter',
            'add_new_item'       => 'Added Filter',
            'edit_item'          => 'Edit Filter',
            'new_item'           => 'New Filter',
            'view_item'          => 'View Filter',
            'search_items'       => 'Search Filter',
            'not_found'          => 'Not found',
            'not_found_in_trash' => 'Not found in trash',
            'parent_item_colon'  => '',
            'menu_name'          => 'Filters',
        ),
            'description'         => 'Filter for your customers',
            'public'              => true,
            'exclude_from_search' => true,
            'publicly_queryable'  => false,
            'show_in_menu'        => false,
            'show_in_admin_bar'   => false,
            'show_in_rest'        => false,
            'menu_position'       => 6,
            'menu_icon'           => 'dashicons-forms',
            'hierarchical'        => false,
            'supports'            => [ 'title' ],
            'taxonomies'          => [],
            'has_archive'         => false,
            'rewrite'             => true,
            'query_var'           => true,
        ) );
        /**
         * Register group for plugins
         */
        register_post_type( 'goon_group', array(
            'label'               => null,
            'labels'              => array(
            'name'               => 'Groups plugins',
            'singular_name'      => 'Group plugins',
            'add_new'            => 'Add Group plugins',
            'add_new_item'       => 'Added Group plugins',
            'edit_item'          => 'Edit Group plugins',
            'new_item'           => 'New Group plugins',
            'view_item'          => 'View Group plugins',
            'search_items'       => 'Search Group plugins',
            'not_found'          => 'Not found',
            'not_found_in_trash' => 'Not found in trash',
            'parent_item_colon'  => '',
            'menu_name'          => 'Groups plugins',
        ),
            'description'         => 'Group plugins for your customers',
            'public'              => true,
            'exclude_from_search' => true,
            'publicly_queryable'  => false,
            'show_in_menu'        => false,
            'show_in_admin_bar'   => false,
            'show_in_rest'        => false,
            'menu_position'       => 6,
            'menu_icon'           => 'dashicons-tickets',
            'hierarchical'        => false,
            'supports'            => [ 'title' ],
            'taxonomies'          => [],
            'has_archive'         => false,
            'rewrite'             => true,
            'query_var'           => true,
        ) );
    }
    
    /**
     * Register all taxonomies
     */
    function register_taxonomies()
    {
        register_taxonomy( 'goon_categories', array( 'goon_filter' ), array(
            'hierarchical' => true,
            'labels'       => array(
            'name'              => _x( 'Categories', 'taxonomy general name' ),
            'singular_name'     => _x( 'Category', 'taxonomy singular name' ),
            'search_items'      => __( 'Search Categories' ),
            'all_items'         => __( 'All Categories' ),
            'parent_item'       => __( 'Parent Category' ),
            'parent_item_colon' => __( 'Parent Category:' ),
            'edit_item'         => __( 'Edit Category' ),
            'update_item'       => __( 'Update Category' ),
            'add_new_item'      => __( 'Create Category' ),
            'new_item_name'     => __( 'New Category Name' ),
            'menu_name'         => __( 'Categories' ),
        ),
            'show_ui'      => true,
            'query_var'    => true,
            'default_term' => 'Uncategorized',
        ) );
    }
    
    /**
     * Disable the dreadful nags
     */
    function disable_all_notice_nags()
    {
        $screen = get_current_screen();
        if ( $screen->base != 'toplevel_page_goon' && strpos( $screen->base, "goon_page_goon" ) === false ) {
            return;
        }
        // goon_mu_plugin()->write_log( $screen->base, "disable_all_notice_nags-screen-base" );
        remove_all_actions( 'admin_notices' );
        remove_all_actions( 'all_admin_notices' );
    }
    
    function is_freemius_blocking_goon_submenu_items()
    {
        global  $submenu ;
        return empty($submenu["goon"]);
        if ( empty($submenu) ) {
            goon_mu_plugin()->write_log( "No submenu found!", "is_freemius_blocking_goon_submenu_items-submenu_missing" );
        }
        goon_mu_plugin()->write_log( $submenu, "is_freemius_blocking_goon_submenu_items-submenu" );
    }

}