<?php
/**
 * Plugin Name:       Goon MU
 * Plugin URI:        https://getgoon.pro
 * Description:       This MU plugin is required by the Goon plugin. It will be removed upon deactivation.
 * Version:           1.2.9.2
 * Author:            getgoon
 * Author URI:        https://profiles.wordpress.org/getgoon/
 * License:           GPL-2.0+
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.txt
 */

// return;

class GOON_MU {

    public $version                 = "1.2.9.2";

    protected static $instance      = null;

    public $current_url             = false;
    public $wp_relative_url         = false;

    public $goon_plugins            = [];
    protected $goon_pages           = [];
    protected $goon_post_types      = [];
    protected $goon_ajax_actions    = [];

    public $is_goon_default_page    = false;
    public $is_being_filtered       = false;
    public $is_skipped              = false;
    public $is_preview_mode         = false;
    public $is_unfiltered_mode      = false;

    public $all_plugins             = [];
    public $excluded_plugins        = [];
    public $original_active_plugins = [];
    public $filtered_active_plugins = [];
    public $plugins_to_block        = [];
    public $blocked_plugins         = [];
    public $filters_in_use          = [];
    
    public $css_to_block            = [];
    public $js_to_block             = [];
    public $blocked_css             = [];
    public $blocked_js              = [];

    public $start_time              = false;

    private function __construct() {

        $this->start_time = microtime(true);
        
        if( ! empty( $_SERVER["REQUEST_TIME_FLOAT"] ) ){
            
            $this->start_time = $_SERVER["REQUEST_TIME_FLOAT"];
        }

        $this->goon_ajax_actions = [
        
            // WP Core
            'update-plugin',
            
            'goon_save_filter',
            'goon_save_group',
            'goon_save_category',
            'goon_create_category',
            'goon_delete_elements',
            'goon_publish_elements',
            'goon_turn_filter_on',
            'goon_turn_filter_off',
            'goon_export_filters',
            'goon_import_filters',
            'goon_mark_tab_complete',
            'goon_save_option_alphabetize_menu',
            'goon_turn_off_filter',
            'goon_save_original_menu',
                // 'goon_get_post_types',// commented out because it wouldn't work!
            'goon_scan_prospector',
            'goon_save_columns_state',
            // 'goon_toggle_exclude_plugin',// commented out because that was the whole purpose of this action
            'goon_include_all_plugins',
            
        ];
        $this->goon_plugins = [
            "goon-plugin-control/goon.php",
            "goon-pro/goon.php",
        ];
        $this->goon_pages = [
            "/wp-admin/admin.php?page=goon",
            "/wp-admin/admin.php?page=goon_filters",
            "/wp-admin/admin.php?page=goon_add_filters",
            "/wp-admin/admin.php?page=goon_filters_categories",
            "/wp-admin/admin.php?page=goon_add_categories",
            "/wp-admin/admin.php?page=goon_groups",
            "/wp-admin/admin.php?page=goon_add_groups",
            "/wp-admin/admin.php?page=goon_worklist",
            "/wp-admin/admin.php?page=goon_settings",
            "/wp-admin/admin.php?page=goon_support",
        ];
        $this->goon_post_types = [
            "goon_filter",
            "goon_group",
            "goon_work",
        ];
        
        $this->excluded_plugins = get_option( "goon_excluded_plugins", [] );
        
        if( $this->should_abort() ){
            return;
        }

        $this->current_full_url         = ( isset( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] === 'on' ? "https" : "http" ) . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
        
        if( ! empty( $_GET["goon_preview_filter"] ) && $_GET["goon_preview_filter"] == "yes" ){
            
            $this->is_preview_mode  = true;
            $this->current_full_url = $this->remove_query_string_parameter( $this->current_full_url, 'goon_preview_filter'  );
            $this->current_full_url = $this->remove_query_string_parameter( $this->current_full_url, 'goon_unfiltered_mode' );
        }
        
        if( ! empty( $_GET["goon_unfiltered_mode"] ) && $_GET["goon_unfiltered_mode"] == "yes" ){
            
            $this->is_unfiltered_mode  = true;
            $this->current_full_url = $this->remove_query_string_parameter( $this->current_full_url, 'goon_preview_filter'  );
            $this->current_full_url = $this->remove_query_string_parameter( $this->current_full_url, 'goon_unfiltered_mode' );
        }
        
        $this->current_wp_relative_url  = str_replace( site_url(), "", $this->current_full_url );

        // $this->write_log( $this->current_full_url, "__construct-current_full_url" );
        // $this->write_log( $this->current_wp_relative_url, "__construct-current_wp_relative_url" );
        
        $this->set_hooks();

    }

    private function should_abort(){

        if( wp_doing_cron() ){

            return true;
        }

        if( defined( 'WP_CLI' ) && WP_CLI ){

            return true;
        }

        if( wp_doing_ajax() && empty( $_POST["action"] ) ){

            return true;
        }

        return false;
    }

    static function get_instance() {

        if( self::$instance == null ){
            self::$instance = new self();
        }

        return self::$instance;

    }


    function set_hooks() {

        add_filter( 'option_active_plugins', [ $this, 'filter_active_plugins_option_value' ], 5 );

        add_action( 'plugins_loaded',        [ $this, 'complete_action_once_plugins_are_loaded' ], 5 );

        // add_action( 'shutdown',              [ $this, 'update_worklist_if_needed' ] );

    }

    function complete_action_once_plugins_are_loaded(){

        remove_filter('option_active_plugins', [ $this, 'filter_active_plugins_option_value' ], 5 );

    }


    function filter_active_plugins_option_value( $active_plugins ) {

        if( ! empty( $this->all_plugins ) ){
            return $active_plugins;
        }

        require_once ABSPATH . 'wp-admin/includes/plugin.php';

        remove_filter('option_active_plugins', [ $this, 'filter_active_plugins_option_value' ], 5 );
        $this->all_plugins              = get_plugins();
        add_filter( 'option_active_plugins', [ $this, 'filter_active_plugins_option_value' ], 5 );

        $this->original_active_plugins  = $active_plugins;

        $active_plugins_on_menu_save = get_option( "active_plugins_on_menu_save" );

        if( $active_plugins_on_menu_save != $active_plugins ){

            // this will trigger the script that recreates the menu:
            update_option( "active_plugins_on_menu_save", $active_plugins, false );
            $_GET["goon_original_menu"] = "get";

        }
        
        
        // $post_id_by_url = url_to_postid( $this->current_full_url );
        // $this->write_log( $post_id_by_url, "mu_plugin-debug-post_id_by_url" );
        
        
        
        

        $this->plugins_to_block         = $this->get_plugins_to_block_for_current_url();




        
        $debug_plugins = "";
        foreach( $this->plugins_to_block as $key => $plugin_id ){
            
            // if( strpos( $plugin_id, "mm" ) === 0 ){
                
                // unset( $this->plugins_to_block[ $key ] );
            // }
            
            if( in_array( $plugin_id, $this->excluded_plugins ) ){
                
                unset( $this->plugins_to_block[ $key ] );
            }
            
            $debug_plugins .= '"' . $plugin_id . '",' . PHP_EOL;
            
        }
        // $this->write_log( $debug_plugins, "mu_plugin-get_plugins_to_block_for_current_url-debug_plugins" );





        $this->filtered_active_plugins  = array_diff( $this->original_active_plugins, $this->plugins_to_block );

        $this->blocked_plugins          = array_intersect( $this->original_active_plugins, $this->plugins_to_block );

        // $this->write_log( $this->plugins_to_block, "mu_plugin-filter_active_plugins_option_value-plugins_to_block" );
        // $this->write_log( $this->blocked_plugins, "mu_plugin-filter_active_plugins_option_value-blocked_plugins" );
        // $this->write_log( $this->filtered_active_plugins, "mu_plugin-filter_active_plugins_option_value-filtered_active_plugins" );
        
        if( ! empty( $this->css_to_block ) && is_array( $this->css_to_block ) ){
            
            $action_name = is_admin() ? 'admin_enqueue_scripts' : 'wp_enqueue_scripts';
            
            add_action( $action_name, [ $this, 'block_css_files' ], PHP_INT_MAX );
        }
        
        if( ! empty( $this->js_to_block ) && is_array( $this->js_to_block ) ){
            
            $action_name = is_admin() ? 'admin_enqueue_scripts' : 'wp_enqueue_scripts';
            
            add_action( $action_name, [ $this, 'block_js_files' ], PHP_INT_MAX );
        }
        
        return $this->filtered_active_plugins;
    }

    function should_block_all( $url ) {

        if( strpos( $url, 'plugin-install.php?tab=plugin-information&plugin=' ) !== false ){
            return true;
        }

    }

    function should_skip_url( $url ) {

        $skip_for = [
            '/favicon.ico',
        ];

        if( in_array( $url, $skip_for ) ){
            return true;
        } elseif( strpos( $url, 'wp-content/plugins' ) !== false ){
            return true;
        } elseif( strpos( $url, 'wp-content/themes' ) !== false ){
            return true;
        } elseif( strpos( $url, '/wp-cron.php' ) !== false ){
            return true;
        } elseif( strpos( $url, '/wp-json/' ) !== false ){
            return true;
        }

        return false;
    }

    function get_plugins_to_block_for_current_url() {


        // On Goon Ajax requests we are blocking all plugins, except Goon
        if( wp_doing_ajax() && in_array( $_POST["action"], $this->goon_ajax_actions ) ){
            
            $block_plugins = array_diff( $this->original_active_plugins, $this->goon_plugins );
            
            // $this->write_log( $_POST, "get_plugins_to_block_for_current_url-post" );
            if( ! empty( $_POST["action"] ) && $_POST["action"] == "update-plugin" && ! empty( $_POST["plugin"] ) ){
                // $this->write_log( $_POST["plugin"], "get_plugins_to_block_for_current_url-update_plugin-post-plugin" );
                
                // disabled this because some plugins need their dependency plugins
                // $block_plugins = array_diff( $block_plugins, [ $_POST["plugin"] ] );
            }
            
            $this->is_skipped = true;
            return $block_plugins;
        }

        // On other Ajax requests maybe we are blocking some plugins
        if( wp_doing_ajax() ){
            
            $filters = get_posts([
                'post_type'     => 'goon_filter',
                'meta_key'      => 'filter_type',
                'meta_value'    => '_ajax_action',
                'numberposts'   => -1,
            ]);

            foreach( $filters as $filter ){

                if( $filter->turned_off ){

                    continue;
                }

                // Filter by Ajax action name

                $ajax_actions = is_array( $filter->ajax_actions ) ? $filter->ajax_actions : [ $filter->ajax_actions ];

                if( in_array( $_POST["action"], $ajax_actions ) ){

                    $this->use_filter( $filter );

                } else {

                    foreach( $ajax_actions as $ajax_action ){
                        
                        if( $this->check_match( $ajax_action, $_POST["action"] ) ){

                            $this->use_filter( $filter );

                            break;
                        }

                    }

                }

            }
            
            // TODO - test if the ajax request is indeed getting filtered or not.
            // $this->write_log( ( $this->is_being_filtered ? "Yes" : "No" ), "get_plugins_to_block_for_current_url-wp_doing_ajax:" . $_POST["action"], "test.log" );
            
            return array_unique( $this->plugins_to_block );
        }


        // some URLs just need all plugins to get blocked
        if( $this->should_block_all( $this->current_wp_relative_url ) ){
            $this->is_skipped = true;
            return $this->original_active_plugins;
        }

        // some URLs just need to be skipped
        if( $this->should_skip_url( $this->current_wp_relative_url ) ){
            $this->is_skipped = true;
            return [];
        }


        $editing_post_type = $this->is_editing_post_type( $this->current_wp_relative_url );

        // --- are we on any of the Goon default pages?
        if(
            strpos( $this->current_wp_relative_url, "wp-admin/admin.php?page=goon") !== false ||
            in_array( $this->current_wp_relative_url, $this->goon_pages ) ||
            in_array( $editing_post_type, $this->goon_post_types )
        ){

            $this->is_goon_default_page = true;
        }


        // when we want to temporarily disable blocking on the current page, we use ?goon_unfiltered_mode=yes on any page
        if( $this->is_unfiltered_mode ){
            $this->is_skipped = true;
            return [];
        }

        // when we are recreating the menu
        if( ! empty( $_GET["goon_original_menu"] ) && $_GET["goon_original_menu"] == "get" ){
            $this->is_skipped = true;
            return [];
        }

        // --- are we on any of the Goon default pages?
        if( $this->is_goon_default_page ){

            $this->is_being_filtered    = true;
            $this->plugins_to_block     = array_diff( $this->original_active_plugins, $this->goon_plugins );

            return $this->plugins_to_block;
        }

        // --- Get plugins to block from all the filters

        $filters = get_posts([
            'post_type'     => 'goon_filter',
            'meta_key'      => 'filter_type',
            'meta_compare'  => '!=',
            'meta_value'    => '_ajax_action',
            'numberposts'   => -1,
        ]);
        
        foreach( $filters as $filter ){

            // $this->write_log( $_GET, "get_plugins_to_block_for_current_url-_GET", true );
            
            if( $filter->turned_off && ! $this->is_preview_mode ){

                continue;
            }
            
            // $this->write_log( $filter, "get_plugins_to_block_for_current_url-filter" );
            // $css_files = get_post_meta( $filter->ID, "css_files", true );
            // $css_files = $filter->css_files;
            // $this->write_log( $css_files, "get_plugins_to_block_for_current_url-css_files" );
            
            
            // If we're on the edit post screen, filter by post type

            if( $editing_post_type && ! in_array( $filter->filter_type, [ '_endpoint', '_endpoint_excluded' ] ) ){
                
                if( $editing_post_type == $filter->filter_type ){

                    $this->use_filter( $filter );
                    
                    if( ! empty( $filter->css_files ) ){ $this->css_to_block = array_merge( $this->css_to_block, $filter->css_files ); }
                    if( ! empty( $filter->js_files  ) ){ $this->js_to_block  = array_merge( $this->js_to_block,  $filter->js_files  ); }

                }
                
                continue;
            }

            // Filter by URL - at this point, only _endpoint and _endpoint_excluded

            $endpoints  = is_array( $filter->endpoints ) ? $filter->endpoints : [ $filter->endpoints ];
            $is_or_nots = is_array( $filter->is_or_not ) ? $filter->is_or_not : [ $filter->is_or_not ];
            
            // $this->write_log( str_repeat( "_", 42 ),  "get_plugins_to_block_for_current_url: " . $filter->post_title );
            // $this->write_log( $endpoints,  "get_plugins_to_block_for_current_url-endpoints: "  . $filter->post_title  );
            // $this->write_log( $is_or_nots, "get_plugins_to_block_for_current_url-is_or_nots: " . $filter->post_title );
            
            $passes_is      = false;
            $passes_is_not  = true;
            
            foreach( $endpoints as $index => $endpoint ){
                
                $is_or_not = ! empty( $is_or_nots[ $index ] ) ? $is_or_nots[ $index ] : "is";
                
                $check_match = $this->check_match( $endpoint, $this->current_wp_relative_url );
                
                // $this->write_log( $check_match, "get_plugins_to_block_for_current_url-check_match: " . $endpoint );
                
                if( $check_match ){
                    
                    if( $is_or_not === "is" ){
                        
                        $passes_is = true;
                        
                    } else {
                        
                        $passes_is_not = false;
                        break;
                    }
                    
                }
                
            }
            
            $use_this_filter = $passes_is && $passes_is_not;
            
            $use_this_filter = $filter->filter_type === '_endpoint_excluded' ? ! $use_this_filter : $use_this_filter;

            if( $use_this_filter ){
                
                $this->use_filter( $filter );

                if( ! empty( $filter->css_files ) ){ $this->css_to_block = array_merge( $this->css_to_block, $filter->css_files ); }
                if( ! empty( $filter->js_files  ) ){ $this->js_to_block  = array_merge( $this->js_to_block,  $filter->js_files  ); }

            }

        }
        
        // $this->write_log( $this->css_to_block, "get_plugins_to_block_for_current_url-this->css_to_block" );
        // $this->write_log( $this->js_to_block,  "get_plugins_to_block_for_current_url-this->js_to_block"  );
        
        return array_unique( $this->plugins_to_block );
    }

    function use_filter( $filter ){

        $this->is_being_filtered = true;

        $plugins_to_block = ! empty( $filter->plugins_to_block ) ? array_keys( $filter->plugins_to_block ) : [];

        $this->plugins_to_block = array_merge( $this->plugins_to_block, $plugins_to_block );

        $this->filters_in_use[ $filter->ID ] = $filter->post_title;

        return $plugins_to_block;

    }


    function block_css_files(){
        
        if( empty( $this->css_to_block ) || ! is_array( $this->css_to_block ) ){
            return;
        }
        
        global $wp_styles;
        
        foreach( $wp_styles->queue as $style_handle ){
            
            $block_style = false;
            
            // $this->write_log( $style_handle, "mu_plugin-block_css_files-style_handle" );
            // $this->write_log( $wp_styles->registered[ $style_handle ], "mu_plugin-block_css_files-wp_styles->registered" );
            
            // $style_handle = $wp_styles->registered[ $style_handle ]->handle;// TO DO is this needed?
            $style_src    = $wp_styles->registered[ $style_handle ]->src;
            
            if( in_array( $style_handle, $this->css_to_block ) ){
                
                $block_style = true;
                
            } else {
                
                foreach( $this->css_to_block as $filename ){
                    
                    if( strpos( $style_src, $filename ) !== false || strpos( $style_handle, $filename ) !== false  ){
                        
                        $block_style = true;
                        
                        break;
                    }
                    
                }
                
            }
            
            if( $block_style ){
                
                wp_dequeue_style( $style_handle );
                
                $this->blocked_css[ $style_handle ] = $style_src;
            }
            
            
        }
        
    }
    function block_js_files(){
        
        if( empty( $this->js_to_block ) || ! is_array( $this->js_to_block ) ){
            return;
        }
        
        global $wp_scripts;
        
        foreach( $wp_scripts->queue as $script_handle ){
            
            $block_script = false;
            
            // $this->write_log( $script_handle, "mu_plugin-block_js_files-script_handle" );
            // $this->write_log( $wp_scripts->registered[ $script_handle ], "mu_plugin-block_js_files-wp_scripts->registered" );
            
            // $script_handle = $wp_scripts->registered[ $script_handle ]->handle;// TO DO is this needed?
            $script_src    = $wp_scripts->registered[ $script_handle ]->src;
            
            if( in_array( $script_handle, $this->js_to_block ) ){
                
                $block_script = true;
                
            } else {
                
                foreach( $this->js_to_block as $filename ){
                    
                    if( strpos( $script_src, $filename ) !== false || strpos( $script_handle, $filename ) !== false  ){
                        
                        $block_script = true;
                        
                        break;
                    }
                    
                }
                
            }
            
            if( $block_script ){
                
                wp_dequeue_script( $script_handle );
                
                $this->blocked_js[ $script_handle ] = $script_src;
            }
            
            
        }
        
    }


    function update_worklist_if_needed(){

        if( $this->is_skipped === false && $this->is_being_filtered === false && ! $this->is_goon_default_page ){

            if( ! is_admin() ){


                // TODO we need to add endpoints to the Worklist here

            }

            // $this->write_log( ( is_admin() ? "Back end" : "Front end" ) . ": " . var_export( trim( $this->current_wp_relative_url ), true ), "update_worklist_if_needed-REQUEST_URI" );
        }

    }


    function is_editing_post_type( $url ){

        $post_id   = $this->url_to_postid( $url );
        $post_type = false;

        if( $post_id !== 0 && strpos( $url, "post.php" ) !== false && strpos( $url, "action=edit" ) !== false ){

            $post_type = get_post_type( $post_id );
        }

        return $post_type;

    }

    function url_to_postid( $url ){

        parse_str( parse_url( $url, PHP_URL_QUERY ), $query_vars);

        $post_id =                   ! empty( $query_vars["post"] )    ? $query_vars["post"]    : 0;
        $post_id = $post_id === 0 && ! empty( $query_vars["post_id"] ) ? $query_vars["post_id"] : $post_id;

        return $post_id;

    }

    function write_log( $log, $text = "write_log: ", $file_name = "debug.log" )  {

        $file = WP_CONTENT_DIR . '/' . $file_name;

        if ( is_array( $log ) || is_object( $log ) ) {
            error_log( $text . PHP_EOL . print_r( $log, true ) . PHP_EOL, 3, $file );
        } else {
            error_log( $text . PHP_EOL . $log . PHP_EOL . PHP_EOL, 3, $file );
        }

    }

    function get_names_list( $array_name, $key = "Name" ){

        $list = [];

        foreach( $this->$array_name as $plugin_id ){

            if( ! empty( $this->all_plugins[ $plugin_id ] ) ){

                $list[ $plugin_id ] = $this->all_plugins[ $plugin_id ][ $key ];
            }

        }

        // $list = array_map( function( $plugin_id ) use ( $key ){

            // return $this->all_plugins[ $plugin_id ][ $key ];

        // }, $this->$array_name );

        natcasesort( $list );

        return $list;

    }
    
    function check_match( $pattern, $text ){
        
        if( strpos( $pattern, "*" ) === false && strpos( $pattern, "+" ) === false ){
            
            return $pattern == $text;
        }
        
        $regex = $pattern;
        $regex = preg_quote( $regex );
        
        $regex = str_replace( '/', '\/', $regex );
        $regex = str_replace( "\*", "(?:.*)", $regex );
        $regex = str_replace( "\+", "(?:.+)", $regex );
        
        $regex = '/' . $regex . '/i';
        
        $found = [];
        preg_match( $regex, $text, $found );
        
        return ! empty( $found[0] ) ? $found[0] == $text : false;
    }
    
    function remove_query_string_parameter( $url, $variable ){
        
        $parsed_url = parse_url( $url );
        $query      = array();

        if( isset( $parsed_url['query'] ) ){
            parse_str( $parsed_url['query'], $query );
            unset( $query[ $variable ] );
        }

        $path  = isset( $parsed_url['path'] ) ? $parsed_url['path'] : '';
        $query = ! empty($query) ? '?'. http_build_query( $query ) : '';

        return $parsed_url['scheme'] . '://' . $parsed_url['host'] . $path . $query;
    }
    
}

function goon_mu_plugin(){
    return GOON_MU::get_instance();
}
goon_mu_plugin();
