<?php

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 */
class Goon
{
    /**
     * The unique identifier of this plugin.
     *
     * @access   protected
     * @var      string $plugin_name The string used to uniquely identify this plugin.
     */
    protected  $plugin_name ;
    /**
     * The current version of the plugin.
     *
     * @access   protected
     * @var      string $version The current version of the plugin.
     */
    protected  $version ;
    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     *
     */
    public function __construct()
    {
        $this->version = GOON_VERSION;
        $this->plugin_name = 'goon';
        $this->include_dependencies();
        $this->init();
    }
    
    /**
     * Load the required dependencies for this plugin.
     *
     * @access   private
     */
    private function include_dependencies()
    {
        // The class responsible for defining internationalization functionality of the plugin.
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-goon-i18n.php';
        // The class responsible for defining helper functionality of the plugin.
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-goon-admin-helper.php';
        // The class responsible for defining all actions that occur in the admin area.
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-goon-admin.php';
        // The class responsible for menu pages of the admin area.
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-goon-admin-pages.php';
        // The class responsible for defining all Ajax actions that occur in the admin area.
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-goon-admin-ajax.php';
        if ( function_exists( "goon_mu_plugin" ) && in_array( "woocommerce/woocommerce.php", goon_mu_plugin()->blocked_plugins ) ) {
            // goon_mu_plugin()->write_log( goon_mu_plugin()->blocked_plugins, "Goon-load_dependencies-blocked_plugins" );
            require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-goon-woocommerce.php';
        }
    }
    
    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @access   private
     */
    private function init()
    {
        // $plugin_i18n  = new GOON_i18n();
        $plugin_admin = new GOON_Admin( $this->get_plugin_name(), $this->get_version() );
        $admin_ajax = new GOON_Ajax();
        $menu_pages = new GOON_Admin_Menu_Pages();
        add_action( 'wp_enqueue_scripts', [ $this, 'enqueue_adminbar_scripts' ], 9998 );
    }
    
    function enqueue_adminbar_scripts()
    {
        if ( wp_style_is( "admin-bar" ) ) {
            wp_enqueue_style(
                $this->plugin_name . '-adminbar',
                plugin_dir_url( __DIR__ ) . 'css/adminbar.css',
                array( "admin-bar" ),
                $this->version,
                'all'
            );
        }
    }
    
    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @return    string    The name of the plugin.
     */
    public function get_plugin_name()
    {
        return $this->plugin_name;
    }
    
    /**
     * Retrieve the version number of the plugin.
     *
     * @return    string    The version number of the plugin.
     */
    public function get_version()
    {
        return $this->version;
    }

}